/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */

package org.apache.poi.hslf.record;

import java.util.HashMap;
import java.lang.reflect.Field;

/**
 * List of all known record types in a PowerPoint document.
 * There are two categories of records:
 * <li> PowerPoint records: 0 <= info <= 10002
 * <li> Escher records: info >= 0xF000
 *
 * @author Yegor Kozlov
 */
public class RecordTypes {
    public static HashMap types;

    public static final int Unknown = 0;
    public static final int Document = 1000;
    public static final int DocumentAtom = 1001;
    public static final int EndDocument = 1002;
    public static final int Slide = 1006;
    public static final int SlideAtom = 1007;
    public static final int Notes = 1008;
    public static final int NotesAtom = 1009;
    public static final int Environment = 1010;
    public static final int SlidePersistAtom = 1011;
    public static final int SSlideLayoutAtom = 1015;
    public static final int MainMaster = 1016;
    public static final int SSSlideInfoAtom = 1017;
    public static final int SlideViewInfo = 1018;
    public static final int GuideAtom = 1019;
    public static final int ViewInfo = 1020;
    public static final int ViewInfoAtom = 1021;
    public static final int SlideViewInfoAtom = 1022;
    public static final int VBAInfo = 1023;
    public static final int VBAInfoAtom = 1024;
    public static final int SSDocInfoAtom = 1025;
    public static final int Summary = 1026;
    public static final int DocRoutingSlip = 1030;
    public static final int OutlineViewInfo = 1031;
    public static final int SorterViewInfo = 1032;
    public static final int ExObjList = 1033;
    public static final int ExObjListAtom = 1034;
    public static final int PPDrawingGroup = 1035;
    public static final int PPDrawing = 1036;
    public static final int NamedShows = 1040;
    public static final int NamedShow = 1041;
    public static final int NamedShowSlides = 1042;
    public static final int SheetProperties = 1044;
    public static final int List = 2000;
    public static final int FontCollection = 2005;
    public static final int BookmarkCollection = 2019;
    public static final int SoundCollAtom = 2021;
    public static final int Sound = 2022;
    public static final int SoundData = 2023;
    public static final int BookmarkSeedAtom = 2025;
    public static final int ColorSchemeAtom = 2032;
    public static final int ExObjRefAtom = 3009;
    public static final int OEShapeAtom = 3009;
    public static final int OEPlaceholderAtom = 3011;
    public static final int GPopublicintAtom = 3024;
    public static final int GRatioAtom = 3031;
    public static final int OutlineTextRefAtom = 3998;
    public static final int TextHeaderAtom = 3999;
    public static final int TextCharsAtom = 4000;
    public static final int StyleTextPropAtom = 4001;
    public static final int BaseTextPropAtom = 4002;
    public static final int TxMasterStyleAtom = 4003;
    public static final int TxCFStyleAtom = 4004;
    public static final int TxPFStyleAtom = 4005;
    public static final int TextRulerAtom = 4006;
    public static final int TextBookmarkAtom = 4007;
    public static final int TextBytesAtom = 4008;
    public static final int TxSIStyleAtom = 4009;
    public static final int TextSpecInfoAtom = 4010;
    public static final int DefaultRulerAtom = 4011;
    public static final int FontEntityAtom = 4023;
    public static final int FontEmbeddedData = 4024;
    public static final int CString = 4026;
    public static final int MetaFile = 4033;
    public static final int ExOleObjAtom = 4035;
    public static final int SrKinsoku = 4040;
    public static final int HandOut = 4041;
    public static final int ExEmbed = 4044;
    public static final int ExEmbedAtom = 4045;
    public static final int ExLink = 4046;
    public static final int BookmarkEntityAtom = 4048;
    public static final int ExLinkAtom = 4049;
    public static final int SrKinsokuAtom = 4050;
    public static final int ExHyperlinkAtom = 4051;
    public static final int ExHyperlink = 4055;
    public static final int SlideNumberMCAtom = 4056;
    public static final int HeadersFooters = 4057;
    public static final int HeadersFootersAtom = 4058;
    public static final int TxInteractiveInfoAtom = 4063;
    public static final int CharFormatAtom = 4066;
    public static final int ParaFormatAtom = 4067;
    public static final int RecolorInfoAtom = 4071;
    public static final int ExQuickTimeMovie = 4074;
    public static final int ExQuickTimeMovieData = 4075;
    public static final int ExControl = 4078;
    public static final int SlideListWithText = 4080;
    public static final int InteractiveInfo = 4082;
    public static final int InteractiveInfoAtom = 4083;
    public static final int UserEditAtom = 4085;
    public static final int CurrentUserAtom = 4086;
    public static final int DateTimeMCAtom = 4087;
    public static final int GenericDateMCAtom = 4088;
    public static final int FooterMCAtom = 4090;
    public static final int ExControlAtom = 4091;
    public static final int ExMediaAtom = 4100;
    public static final int ExVideo = 4101;
    public static final int ExAviMovie = 4102;
    public static final int ExMCIMovie = 4103;
    public static final int ExMIDIAudio = 4109;
    public static final int ExCDAudio = 4110;
    public static final int ExWAVAudioEmbedded = 4111;
    public static final int ExWAVAudioLink = 4112;
    public static final int ExOleObjStg = 4113;
    public static final int ExCDAudioAtom = 4114;
    public static final int ExWAVAudioEmbeddedAtom = 4115;
    public static final int AnimationInfoAtom = 4116;
    public static final int RTFDateTimeMCAtom = 4117;
    public static final int ProgTags = 5000;
    public static final int ProgStringTag = 5001;
    public static final int ProgBinaryTag = 5002;
    public static final int BinaryTagData = 5003;
    public static final int PrpublicintOptions = 6000;
    public static final int PersistPtrFullBlock = 6001;
    public static final int PersistPtrIncrementalBlock = 6002;
    public static final int GScalingAtom = 10001;
    public static final int GRColorAtom = 10002;

    //records greater then 0xF000 belong to with Microsoft Office Drawing format also known as Escher
    public static final int EscherDggContainer = 0xf000;
    public static final int EscherDgg = 0xf006;
    public static final int EscherCLSID = 0xf016;
    public static final int EscherOPT = 0xf00b;
    public static final int EscherBStoreContainer = 0xf001;
    public static final int EscherBSE = 0xf007;
    public static final int EscherBlip_START = 0xf018;
    public static final int EscherBlip_END = 0xf117;
    public static final int EscherDgContainer = 0xf002;
    public static final int EscherDg = 0xf008;
    public static final int EscherRegroupItems = 0xf118;
    public static final int EscherColorScheme = 0xf120;
    public static final int EscherSpgrContainer = 0xf003;
    public static final int EscherSpContainer = 0xf004;
    public static final int EscherSpgr = 0xf009;
    public static final int EscherSp = 0xf00a;
    public static final int EscherTextbox = 0xf00c;
    public static final int EscherClientTextbox = 0xf00d;
    public static final int EscherAnchor = 0xf00e;
    public static final int EscherChildAnchor = 0xf00f;
    public static final int EscherClientAnchor = 0xf010;
    public static final int EscherClientData = 0xf011;
    public static final int EscherSolverContainer = 0xf005;
    public static final int EscherConnectorRule = 0xf012;
    public static final int EscherAlignRule = 0xf013;
    public static final int EscherArcRule = 0xf014;
    public static final int EscherClientRule = 0xf015;
    public static final int EscherCalloutRule = 0xf017;
    public static final int EscherSelection = 0xf119;
    public static final int EscherColorMRU = 0xf11a;
    public static final int EscherDeletedPspl = 0xf11d;
    public static final int EscherSplitMenuColors = 0xf11e;
    public static final int EscherOleObject = 0xf11f;
    public static final int EscherUserDefined = 0xf122;

    /**
     * Returns name of the record by its type
     *
     * @param type section of the record header
     * @return name of the record
     */
    public static String recordName(int type) {
        String name = (String)types.get(new Integer(type));
        if (name == null) name = "Unknown" + type;
        return name;
    }

    static {
        types = new HashMap();
        try {
            Field[] f = RecordTypes.class.getFields();
            for (int i = 0; i < f.length; i++){
                Object val = f[i].get(null);
                if (val instanceof Integer) {
                    types.put(val, f[i].getName());
                }
            }
        } catch (IllegalAccessException e){
            throw new RuntimeException("Failed to initialize records types");
        }
    }

}
